package apap.tutorial.belajarbelajar.restcontroller;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.NoSuchElementException;

import javax.validation.Valid;

import apap.tutorial.belajarbelajar.rest.CourseDetail;
import apap.tutorial.belajarbelajar.rest.PengajarDetail;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.server.ResponseStatusException;

import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.service.PengajarRestService;
import reactor.core.publisher.Mono;

@RestController
@RequestMapping("/api/v1")
public class PengajarRestController {
    @Autowired
    private PengajarRestService pengajarRestService;

    // Add
    @PostMapping(value = "/pengajar")
    private PengajarModel createPengajar(@Valid @RequestBody PengajarModel pengajar, BindingResult bindingResult) {
        if (bindingResult.hasFieldErrors()) {
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST, "Request body has invalid type or missing field."
            );
        } else {
            return pengajarRestService.createPengajar(pengajar);
        }
    }

    // Retrieve
    @GetMapping(value = "/{noPengajar}")
    private PengajarModel retrievePengajar(@PathVariable("noPengajar") Long noPengajar) {
        try {
            return pengajarRestService.getPengajarByNoPengajar(noPengajar);
        } catch (NoSuchElementException e) {
            throw new ResponseStatusException(
                    HttpStatus.NOT_FOUND, "Pengajar " + noPengajar + " not found."
            );
        }
    }

    // Delete
    @DeleteMapping(value = "/pengajar/{noPengajar}")
    private ResponseEntity deletePengajar(@PathVariable("noPengajar") Long noPengajar) {
        try {
            pengajarRestService.deletePengajar(noPengajar);
            return ResponseEntity.ok("Pengajar with noPengajar " + noPengajar + " has been deleted successfully.");
        } catch (NoSuchElementException e) {
            throw new ResponseStatusException(
                    HttpStatus.NOT_FOUND, "Pengajar " + noPengajar + " not found."
            );
        } catch (UnsupportedOperationException e) {
            throw new ResponseStatusException(
                    HttpStatus.BAD_REQUEST, "Bad Request"
            );
        }
    }

    // Update
    @PutMapping(value = "/pengajar/{noPengajar}")
    private PengajarModel updateCourse(@PathVariable("noPengajar") Long noPengajar, @RequestBody PengajarModel pengajar) {
        try {
            return pengajarRestService.updatePengajar(noPengajar, pengajar);
        } catch (NoSuchElementException e) {
            throw new ResponseStatusException(
                    HttpStatus.NOT_FOUND, "noPengajar " + noPengajar + " not found"
            );
        }
    }

    // Retrieve List All
    @GetMapping(value = "/list-pengajar")
    private List<PengajarModel> retrieveListPengajar() {
        return pengajarRestService.retrieveListPengajar();
    }

    @GetMapping(value = "/pengajar/jenis-kelamin/{noPengajar}")
    private PengajarModel jenisKelaminPengajar(@PathVariable("noPengajar") Long noPengajar) {
        try {
            return pengajarRestService.prediksiJenisKelaminPengajar(noPengajar);
        } catch (NoSuchElementException e) {
            throw new ResponseStatusException(
              HttpStatus.NOT_FOUND, "No Pengajar " + String.valueOf(noPengajar) + " Not Found."
            );
        }
    }

//    @GetMapping(value="/pengajar/jenis-kelamin/{noPengajar}")
//    private HashMap<String,Object> mapPengajar(@PathVariable("noPengajar")Long noPengajar){
//        return pengajarRestService.simpanJenisKelamin(noPengajar,pengajarRestService.getJenisKelamin(noPengajar).block().getValidUntil().before(now));
//    }

//    @GetMapping(value = "/pengajar/{noPengajar}/status")
//    private Mono<String> getStatus(@PathVariable("noPengajar") Long noPengajar) {
//        return pengajarRestService.getStatus(noPengajar);
//    }
//
//    @GetMapping(value = "/full")
//    private Mono<PengajarDetail> postStatus() {
//        return pengajarRestService.postStatus();
//    }
}
