package apap.tutorial.belajarbelajar.service;

import java.time.LocalDate;
import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;
import java.util.NoSuchElementException;
import java.util.Optional;

import javax.transaction.Transactional;

import apap.tutorial.belajarbelajar.repository.PengajarDb;
import apap.tutorial.belajarbelajar.rest.CourseDetail;
import apap.tutorial.belajarbelajar.rest.JenisKelaminPengajar;
import apap.tutorial.belajarbelajar.rest.PengajarDetail;
import apap.tutorial.belajarbelajar.rest.Setting;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import apap.tutorial.belajarbelajar.model.PengajarModel;
import apap.tutorial.belajarbelajar.repository.PengajarDb;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.reactive.function.client.WebClient;
import reactor.core.publisher.Mono;

@Service
@Transactional
public class PengajarRestServiceImpl implements PengajarRestService {

    private final WebClient webClient;

    public PengajarRestServiceImpl(WebClient.Builder webClientBuilder) {
        this.webClient = webClientBuilder.baseUrl("https://api.genderize.io").build();
    }
    @Autowired
    private PengajarDb pengajarDb;

    @Override
    public PengajarModel createPengajar(PengajarModel pengajar) {
        return pengajarDb.save(pengajar);
    }

    @Override
    public List<PengajarModel> retrieveListPengajar() {
        return pengajarDb.findAll();
    }

    @Override
    public PengajarModel getPengajarByNoPengajar(Long noPengajar) {
        Optional<PengajarModel> pengajar = pengajarDb.findByNoPengajar(noPengajar);
        if (pengajar.isPresent()) {
            return pengajar.get();
        } else {
            throw new NoSuchElementException();
        }
    }

    @Override
    public PengajarModel updatePengajar(Long noPengajar, PengajarModel pengajarUpdate) {
        PengajarModel pengajar = getPengajarByNoPengajar(noPengajar);
        pengajar.setNamaPengajar(pengajarUpdate.getNamaPengajar());
        pengajar.setCourse(pengajarUpdate.getCourse());
        pengajar.setIsPengajarUniversitas(pengajarUpdate.getIsPengajarUniversitas());
        return pengajarDb.save(pengajar);
    }

    @Override
    public void deletePengajar(Long noPengajar) {
        PengajarModel pengajar = getPengajarByNoPengajar(noPengajar);
        try {
            pengajarDb.delete(pengajar);
        } catch (Exception e) {
            throw new UnsupportedOperationException();
        }
    }

    @Override
    public PengajarModel prediksiJenisKelaminPengajar(Long noPengajar) {
        LocalDateTime sekarang = LocalDateTime.now();
        PengajarModel pengajar = getPengajarByNoPengajar(noPengajar);

        String namaPengajar = pengajar.getNamaPengajar();
        if (namaPengajar.length() > 1) {
            String[] namaDepanPengajar = namaPengajar.split(" ");
            namaPengajar = namaDepanPengajar[0];
        }

        if (sekarang.isBefore(pengajar.getCourse().getTanggalBerakhir()) && sekarang.isAfter(pengajar.getCourse().getTanggalDimulai())) {
            throw new UnsupportedOperationException("Course sedang dibuka!");
        } else {
            Mono<JenisKelaminPengajar> jenisKelaminPengajar = this.webClient.get().uri("?name=" + namaPengajar)
                    .retrieve()
                    .bodyToMono(JenisKelaminPengajar.class);

            if (jenisKelaminPengajar.block().getGender().equals("male")) {
                pengajar.setJenisKelamin(false);
            } else {
                pengajar.setJenisKelamin(true);
            }

            pengajarDb.save(pengajar);
            return pengajar;
        }
    }

//    @Override
//    public Mono<PengajarDetail> getJenisKelamin(Long noPengajar){
//        String namaPengajar = pengajarDb.findByNoPengajar(noPengajar).get().getNamaPengajar();
//        return this.webClient.get().uri("https://api.genderize.io?name="+ namaPengajar.split(" ")[0])
//                .retrieve()
//                .bodyToMono(PengajarDetail.class);
//    }
//
//    @Override
//    public HashMap<String,Object> simpanJenisKelamin(Long noPengajar, Boolean jenisKelamin){
//        PengajarModel pengajar = pengajarDb.findByNoPengajar(noPengajar);
//        pengajar.setJenisKelamin(jenisKelamin);
//        pengajarDb.save(pengajar);
//
//        HashMap<String,Object> map = new HashMap<>();
//        map.put("noPengajar",pengajar.getNoPengajar());
//        map.put("namaPengajar",pengajar.getNamaPengajar().split(" ")[0]);
//        map.put("isPengajarUniversitas",pengajar.getIsPengajarUniversitas());
//        map.put("jenisKelamin",pengajar.getJenisKelamin());
//
//        return map;
//    }


//    @Override
//    public Mono<String> getStatus(Long noPengajar) {
//        return this.webClient.get().uri("/rest/course/" + noPengajar + "/status").retrieve().bodyToMono(String.class);
//    }
//
//    @Override
//    public Mono<PengajarDetail> postStatus() {
//        MultiValueMap<String, String> data = new LinkedMultiValueMap<>();
////        data.add("code", "APAP");
////        data.add("nameCourse", "Arsitektur PAP");
//        return this.webClient.post().uri("/rest/pengajar/full").syncBody(data).retrieve().bodyToMono(PengajarDetail.class);
//    }
}
